@@ -6,10 +6,29 @@
 # =====================================================================================================================
 
 class SeeqNames:
+    class Database:
+        class Schemas:
+            materialized_tables = 'materialized_tables'
+        
+    
+    class MaterializedTables:
+        datum_id_column = 'datum id'
+        item_id_column = 'item id'
+        event_data_column = 'event data'
+        context_table_prefix = 'context_'
+        class Rules:
+            ancestor = 'ancestor'
+            concat_columns = 'concatColumns'
+            event_property = 'eventProperty'
+            item_property = 'itemProperty'
+            path = 'path'
+        
+    
     class Injection:
         everyone_user_group = 'EveryoneUserGroup'
         system_state = 'SystemState'
         system_user = 'SystemUser'
+        agents_user_group = 'AgentsUserGroup'
     
     class Connectors:
         seeq_developer_name = 'Seeq'
@@ -67,6 +86,7 @@ class SeeqNames:
     
     class OData:
         export_entity_set_suffix = '_DataSet'
+        o_data_v4_service_suffix = '.svc/'
     
     class CleanUpJobsGroup:
         annotation_images = 'AnnotationOrphanedImageCleanerJob'
@@ -116,11 +136,13 @@ class SeeqNames:
         file_folders = 'FileFolders'
         g_e_proficy = 'GE Proficy'
         i_b_a_h_d = 'IbaHD'
+        ignition = 'Ignition Gateway'
         influx_d_b = 'InfluxDB'
         influx_d_b2 = 'InfluxDB 2'
         i_p21 = 'AspenTech IP21'
         met_net = 'MetNet'
         n_o_a_a_weather = 'NOAA Weather Service'
+        omnia = 'Equinor Omnia Plant'
         o_p_c_h_d_a = 'OPC-HDA'
         o_p_c_u_a = 'OPC-UA'
         o_s_isoft_a_f = 'OSIsoft AF'
@@ -134,6 +156,12 @@ class SeeqNames:
         wonderware = 'Wonderware'
         x_h_q = 'XHQ'
     
+    class IdentityProviderDatasourceClasses:
+        l_d_a_p = 'LDAP'
+        o_auth2 = 'OAuth 2.0'
+        seeq_auth = 'Auth'
+        windows_auth = 'Windows Auth'
+    
     class LocalDatasources:
         class SeeqMetadataItemStorage:
             datasource_class = 'Seeq Metadata'
@@ -216,6 +244,10 @@ class SeeqNames:
         displays = 'Displays'
         display_templates = 'DisplayTemplates'
         notification_configurations = 'NotificationConfigurations'
+        label = 'Label'
+        context = 'Context'
+        table_definitions = 'TableDefinitions'
+        graph_q_l = 'GraphQL'
     
     class Channels:
         agents_status = 'agents-status'
@@ -238,6 +270,7 @@ class SeeqNames:
         folders = '/folders'
         networks = '/networks'
         items = '/items'
+        item_finders = '/finders'
         reports = '/reports'
         jobs = '/jobs'
         logs = '/logs'
@@ -271,9 +304,11 @@ class SeeqNames:
         user_groups = '/usergroups'
         access_keys = '/accesskeys'
         o_data = '/odata.svc'
+        o_data_v4 = '/odata'
         system = '/system'
         import_ = '/import'
         status = '/status'
+        shutdown = '/shutdown'
         screenshots = '/screenshots'
         subscriptions = '/subscriptions'
         monitors = '/system/monitors'
@@ -289,6 +324,13 @@ class SeeqNames:
         display_templates = '/display-templates'
         usage = '/usage'
         notification_configurations = '/notification-configurations'
+        labels = '/labels'
+        context = '/context'
+        context_comments = '/comments'
+        context_labels = '/labels'
+        context_opaque = '/opaque'
+        table_definitions = '/table-definitions'
+        graph_q_l = '/graphql'
         class ErrorMessages:
             attempted_to_set_scope_on_a_globally_scoped_item = 'Attempted to set scope on a globally scoped item'
         
@@ -312,12 +354,17 @@ class SeeqNames:
             request_id = 'requestId'
         
         class Headers:
-            agent_id = 'x-sq-agent-id'
+            accept_ranges = 'Accept-Ranges'
             async_ = 'x-sq-async'
+            async_job = 'x-sq-async-job'
             auth = 'x-sq-auth'
+            identity_path = 'x-sq-identity-path'
+            content_range = 'Content-Range'
             csrf = 'x-sq-csrf'
             forwarded_for = 'x-forwarded-for'
             interactive_session_id = 'Session-Id'
+            pool_id = 'x-sq-pool-id'
+            range = 'Range'
             request_id = 'x-sq-request-id'
             request_origin = 'x-sq-origin'
             request_origin_u_r_l = 'x-sq-origin-url'
@@ -349,6 +396,11 @@ class SeeqNames:
                 database_relationships_read = 'Database Relationships Read'
             
         
+        class Flags:
+            all_properties = '@allProperties'
+            exclude_globally_scoped = '@excludeGloballyScoped'
+            include_unsearchable = '@includeUnsearchable'
+        
     
     class UnitTest:
         timestamp1 = '1990-03-07T09:58:20.888888Z'
@@ -399,11 +451,13 @@ class SeeqNames:
         has_asset_selections = 'has asset selections'
         has_condition = 'has condition'
         monitors = 'monitors'
+        has_item_finder = 'has item finder'
         has_content = 'has content'
         has_date_range = 'has date range'
         has_date_ranges = 'has date ranges'
         has_scheduled_notebook = 'has scheduled notebook'
         has_template = 'has template'
+        subscribed_to = 'subscribed to'
     
     class Types:
         asset = 'Asset'
@@ -411,8 +465,6 @@ class SeeqNames:
         access_control_entry = 'AccessControlEntry'
         stored_signal = 'StoredSignal'
         datasource = 'Datasource'
-        # DEPRECATED, Deprecated. See SeeqNames.java for more info
-        table_datasource = 'TableDatasource'
         datafile = 'Datafile'
         calculated_signal = 'CalculatedSignal'
         signal = 'Signal'
@@ -430,11 +482,14 @@ class SeeqNames:
         report_template = 'ReportTemplate'
         journal = 'Journal'
         reply = 'Reply'
+        item_finder = 'ItemFinder'
         condition = 'Condition'
         condition_monitor = 'ConditionMonitor'
         capsule = 'Capsule'
         stored_condition = 'StoredCondition'
         calculated_condition = 'CalculatedCondition'
+        scalar = 'ScalarItem'
+        literal_scalar = 'LiteralScalar'
         calculated_scalar = 'CalculatedScalar'
         formula_function = 'FormulaFunction'
         aggregating_formula_function = 'AggregatingFormulaFunction'
@@ -466,6 +521,7 @@ class SeeqNames:
         item = 'Item'
         item_with_owner = 'ItemWithOwner'
         type_definition = 'TypeDefinition'
+        table_definition = 'TableDefinition'
     
     class TopicDocumentAttributes:
         data_seeq_content = 'data-seeq-content'
@@ -489,6 +545,8 @@ class SeeqNames:
         start = 'Start'
         end = 'End'
         duration = 'Duration'
+        is_uncertain = 'IsUncertain'
+        is_bounded = 'IsBounded'
         middle = 'Middle'
         count = 'Count'
         value = 'Value'
@@ -519,12 +577,15 @@ class SeeqNames:
         string_format = 'String Format'
         source_string_format = 'Source String Format'
         storage_location = 'Storage Location'
+        executor_id = 'Executor ID'
         creator_id = 'Creator ID'
         creator_first_name = 'Creator First Name'
         creator_last_name = 'Creator Last Name'
         historical_data_version = 'Historical Data Version'
         manual_cdc = 'Manual CDC'
         scoped_to = 'Scoped To'
+        asset = 'asset'
+        full_path = 'fullpath'
         import_item_type = 'Import Item Type'
         filename = 'Filename'
         name_row = 'Name Row'
@@ -574,8 +635,11 @@ class SeeqNames:
         indexing_schedule_supported = 'Indexing Schedule Supported'
         asset_tree_search_index_needs_updating = 'Asset Tree Search Index Needs Updating'
         sync_mode = 'Sync Mode'
+        items_archived_count = 'Items Archived Count'
         indexing_duration = 'Indexing Duration'
-        current_indexing_started_at = 'Current Indexing Started At'
+        provides_everyone_group_identities = 'Provides Everyone Group Identities'
+        datasource_labels = 'Datasource Labels'
+        indexing_progress_timestamp = 'Indexing Progress Timestamp'
         asset_progress = 'Asset Progress'
         asset_count = 'Asset Count'
         previous_asset_count = 'Previous Asset Count'
@@ -617,7 +681,9 @@ class SeeqNames:
         formula = 'Formula'
         swap_source_id = 'Swap Source ID'
         swap_key = 'Swap Key'
+        cache_flush_version = 'Cache Flush Version'
         cache_value = 'CacheValue'
+        value = 'Value'
         formula_parameters = 'FormulaParameters'
         expected_provided_type = 'ExpectedProvidedType'
         version_string = 'Version String'
@@ -665,6 +731,9 @@ class SeeqNames:
         admin = 'Admin'
         workbench = 'Workbench'
         user_language = 'userLanguage'
+        agent_provisioning_status = 'Agent Provisioning Status'
+        agent_provisioning_source_address = 'Agent Provisioning Source Address'
+        agent_provisioning_requested_at = 'Agent Provisioning Requested At'
         remote_group_editable = 'Remote Group Editable'
         remove_permissions = 'Remove Permissions'
         data = 'Data'
@@ -697,10 +766,13 @@ class SeeqNames:
         summary_type = 'Summary Type'
         summary_value = 'Summary Value'
         current_filename = 'Current Filename'
+        last_result_hash = 'Last Result Hash'
         asset_path_depth = 'Asset Path Depth'
         last_run_at = 'Last Run At'
         last_run_state = 'Last Run State'
         query_range_look_ahead = 'Query Range Look Ahead'
+        finder_configurations = 'Finder Configurations'
+        last_run_warnings = 'Last Run Warnings'
         resource_size = 'Resource Size'
         project_type = 'Project Type'
         cron_schedule = 'Cron Schedule'
@@ -710,6 +782,8 @@ class SeeqNames:
         previous_run_time = 'Previous Run Time'
         cron_schedules_to_indices = 'Cron Schedules To Indices'
         label = 'Label'
+        notify_on_skipped_execution = 'Notify On Skipped Execution'
+        notify_on_automatic_unschedule = 'Notify On Automatic Unschedule'
         total_run_time = 'Total Run Time'
         average_run_time = 'Average Run Time'
         is_fixed_with = 'Is Fixed Width'
@@ -751,6 +825,7 @@ class SeeqNames:
         disabled_at = 'Disabled At'
         disabled_by = 'Disabled By'
         indexing_on_startup_and_config_change = 'Indexing On Startup And Config Change'
+        column_definitions = 'Column Definitions'
     
 
 
