@@ -6,6 +6,7 @@ import logging
 import os
 import re
 import warnings
+from dataclasses import dataclass
 from typing import Optional, Tuple, Union
 from urllib.parse import urlparse, quote
 
@@ -19,11 +20,10 @@ from dateutil.tz import tz
 from urllib3.connectionpool import MaxRetryError, SSLError
 
 from seeq import spy, sdk
-from seeq.base import system, util
+from seeq.base import util
 from seeq.sdk import *
 from seeq.spy import _common, _datalab
 from seeq.spy._config import Setting
-from seeq.spy._datalab import is_datalab_functions_project
 from seeq.spy._errors import *
 from seeq.spy._session import Session
 from seeq.spy._status import Status
@@ -31,6 +31,7 @@ from seeq.spy._status import Status
 AUTOMATIC_PROXY_DETECTION = '__auto__'
 
 
+@Status.handle_keyboard_interrupt()
 def login(username=None, password=None, *, access_key=None, url=None, directory='Seeq',
           ignore_ssl_errors=False, proxy=AUTOMATIC_PROXY_DETECTION, credentials_file=None, force=True,
           quiet=None, status=None, session: Session = None, private_url=None, auth_token=None, csrf_token=None,
@@ -93,7 +94,7 @@ def login(username=None, password=None, *, access_key=None, url=None, directory=
         the username, the second being the user's password.
 
     force : str, default True
-        If True, re-logs in even if already already logged in. If False, skips
+        If True, re-logs in even if already logged in. If False, skips
         login if already logged in. You should include a spy.login(force=False)
         cell if you are creating example notebooks that may be used in Jupyter
         environments like Anaconda, AWS SageMaker or Azure Notebooks.)
@@ -101,7 +102,7 @@ def login(username=None, password=None, *, access_key=None, url=None, directory=
     quiet : bool, default False
         If True, suppresses progress output. Note that when status is
         provided, the quiet setting of the Status object that is passed
-        in takes precedent.
+        in takes precedence.
 
     status : spy.Status, optional
         If supplied, this Status object will be updated as the command
@@ -124,12 +125,14 @@ def login(username=None, password=None, *, access_key=None, url=None, directory=
 
     request_origin_label : str
         Used for tracking Data Consumption. If supplied, this label will be added as a header to all requests from
-        the logged in user. Not necessary in Data Lab because the header will already be filled in.
+        the logged in user. Not necessary in Data Lab because the header will already be filled in. You can also specify
+        this value after login by setting the spy.session.request_origin_label property.
 
     request_origin_url : str
         Used for tracking Data Consumption. If supplied, this label will be added as a header to all requests from
         the logged in user. Not necessary in Data Lab because the header will already be filled in. If NOT in Data
-        Lab, supply a full URL that leads to the tool/plugin that is consuming data, if applicable.
+        Lab, supply a full URL that leads to the tool/plugin that is consuming data, if applicable. You can also specify
+        this value after login by setting the spy.session.request_origin_url property.
 
     Examples
     --------
@@ -146,19 +149,21 @@ def login(username=None, password=None, *, access_key=None, url=None, directory=
         (password, 'password', str),
         (access_key, 'access_key', str),
         (url, 'url', str),
-        (private_url, 'private_url', str),
         (directory, 'directory', str),
         (ignore_ssl_errors, 'ignore_ssl_errors', bool),
         (proxy, 'proxy', str),
         (credentials_file, 'credentials_file', str),
+        (force, 'force', bool),
+        (quiet, 'quiet', bool),
+        (status, 'status', Status),
+        (session, 'session', Session),
+        (private_url, 'private_url', str),
         # Note: Although auth_token is no longer a supported authentication method for non-Seeq Data Lab scenarios,
         # it is still used by Seeq Data Lab code to log the user in.
         (auth_token, 'auth_token', str),
         (csrf_token, 'csrf_token', str),
-        (force, 'force', bool),
-        (quiet, 'quiet', bool),
-        (status, 'status', Status),
-        (session, 'session', Session)
+        (request_origin_label, 'request_origin_label', str),
+        (request_origin_url, 'request_origin_url', str),
     ])
 
     status = Status.validate(status, quiet)
@@ -175,24 +180,7 @@ def login(username=None, password=None, *, access_key=None, url=None, directory=
         status.update(str(e), Status.FAILURE)
         raise
 
-    user_string = session.user.username
-    user_profile = ''
-    if session.user.first_name:
-        user_profile = session.user.first_name
-    if session.user.last_name:
-        user_profile += ' ' + session.user.last_name
-    if session.user.is_admin:
-        user_profile += ' [Admin]'
-    if len(user_profile) > 0:
-        user_string += ' (%s)' % user_profile.strip()
-
-    message = f'Logged in to <strong>{session.public_url}</strong> successfully as <strong>{user_string}</strong>.\n'
-
-    message += (f'Seeq Server Version: <strong>{spy.server_version}</strong>\n'
-                f'Seeq SDK Module Version: <strong>{sdk.__version__}</strong> @ {os.path.dirname(sdk.__file__)}\n'
-                f'Seeq SPy Module Version: <strong>{spy.__version__}</strong> @ {os.path.dirname(spy.__file__)}')
-
-    status.update(message, Status.SUCCESS)
+    status.update(session.get_info(html=True), Status.SUCCESS)
 
 
 def _login(username, password, access_key, url, directory, ignore_ssl_errors, proxy, credentials_file, auth_token,
@@ -244,6 +232,18 @@ def _login(username, password, access_key, url, directory, ignore_ssl_errors, pr
 
         folders_api = FoldersApi(session.client)
 
+        default_request_origin = _retrieve_default_request_origin(session)
+        if default_request_origin is not None:
+            if request_origin_label is None:
+                request_origin_label = default_request_origin.label
+            if request_origin_url is None:
+                request_origin_url = default_request_origin.url
+
+        if request_origin_label is not None:
+            session.request_origin_label = request_origin_label
+        if request_origin_url is not None:
+            session.request_origin_url = request_origin_url
+
         # noinspection PyBroadException
         try:
             session.corporate_folder = folders_api.get_folder(folder_id='corporate')
@@ -284,8 +284,12 @@ def _client_login(auth_token, credentials_file, directory, ignore_ssl_errors, pa
     _client = ApiClient(api_client_url, configuration=session.client_configuration)
     if _datalab.is_datalab_api():
         _client.set_default_header('x-sq-origin', 'Add-on')
+    elif _datalab.is_executor():
+        _client.set_default_header('x-sq-origin', 'Data Lab (Job)')
+    elif _datalab.is_datalab():
+        _client.set_default_header('x-sq-origin', 'Data Lab (Interactive)')
     else:
-        _client.set_default_header('x-sq-origin', 'Data Lab')
+        _client.set_default_header('x-sq-origin', 'SPy (Standalone)')
 
     auth_api = AuthApi(_client)
     directories = dict()
@@ -321,8 +325,8 @@ def _client_login(auth_token, credentials_file, directory, ignore_ssl_errors, pa
                                     'access_key and password must be omitted')
 
             if not os.path.exists(credentials_file):
-                repo_root_dir = system.get_test_with_root_dir()
-                if repo_root_dir:
+                repo_root_dir = util.get_test_with_root_dir()
+                if repo_root_dir is not None:
                     agent_key_file = os.path.join(repo_root_dir, 'sq-run-data-dir', 'keys', 'agent.key')
                     if os.path.exists(agent_key_file):
                         # This is a hack to make it easier to run the Documentation notebooks without having to create a
@@ -396,38 +400,8 @@ def _client_login(auth_token, credentials_file, directory, ignore_ssl_errors, pa
     session.https_key_file = key_file
     session.https_cert_file = cert_file
 
-    # Now that we are logged in, retrieve path and url for data consumption
-    if ((_datalab.is_datalab() or _datalab.is_executor())
-            and not _datalab.is_datalab_api()
-            and request_origin_url is None
-            and request_origin_label is None):
-        # noinspection PyBroadException
-        try:
-            if _datalab.is_executor():
-                project_name = os.environ.get('SEEQ_PROJECT_NAME', '')
-                file_path = os.environ.get('SEEQ_SDL_FILE_PATH', '')
-                project_url = _datalab.get_data_lab_project_url(use_private_url=False)
-                notebook_url = f"{project_url}/notebooks/{file_path}"
-                request_origin_label = f"[Scheduled] {project_name} - {file_path}"
-                request_origin_url = quote(urlparse(notebook_url).path)
-            else:
-                notebook_url = _datalab.get_notebook_url(session)
-                request_origin_url = quote(urlparse(notebook_url).path)
-
-                project_name = _datalab.get_data_lab_project_name(session)
-                file_path = notebook_url.split('notebooks/', 1)[1]
-                request_origin_label = project_name + " - " + file_path
-        except BaseException:
-            pass
-
-    if request_origin_label is not None:
-        if is_datalab_functions_project() and not request_origin_label.startswith("[Data Lab Functions]"):
-            request_origin_label = f"[Data Lab Functions] {request_origin_label}"
-        _client.set_default_header('x-sq-origin-label', request_origin_label)
-    if request_origin_url is not None:
-        _client.set_default_header('x-sq-origin-url', request_origin_url)
-
 
+@Status.handle_keyboard_interrupt()
 def logout(quiet=None, status=None, session: Session = None):
     """
     Logs you out of your current session.
@@ -438,7 +412,7 @@ def logout(quiet=None, status=None, session: Session = None):
     quiet : bool, default False
         If True, suppresses progress output. Note that when status is
         provided, the quiet setting of the Status object that is passed
-        in takes precedent.
+        in takes precedence.
 
     status : spy.Status, optional
         If supplied, this Status object will be updated as the command
@@ -468,7 +442,7 @@ def _clear_login_state(quiet, status, session: Session, call_logout=False):
     quiet : bool
         If True, suppresses progress output. Note that when status is
         provided, the quiet setting of the Status object that is passed
-        in takes precedent.
+        in takes precedence.
 
     status : spy.Status, optional
         If supplied, this Status object will be updated as the command
@@ -791,20 +765,9 @@ def validate_seeq_server_version(session: Session, status: Status, allow_version
                        f'({seeq_server_major}) and is incompatible.')
 
     if message is not None:
-        compatible_module_folder = find_compatible_module(session)
-        if not compatible_module_folder:
-            message += (f'\n\nIt is recommended that you run spy.upgrade() or issue the following PIP command to '
-                        f'install a compatible version of the seeq module:\n')
-            if using_old_server_versioning_scheme:
-                message += generate_seeq_module_pip_upgrade(
-                    f'{seeq_server_major}.{seeq_server_minor}.{seeq_server_patch}')
-            else:
-                message += generate_seeq_module_pip_upgrade(f'{seeq_server_major}.{seeq_server_minor}')
-        else:
-            message += (f'\n\nA compatible module was found here: {compatible_module_folder}\n'
-                        f'Consider uninstalling your project-specific seeq module by issuing the following PIP '
-                        f'command:\n'
-                        f'pip uninstall -y seeq')
+        message += (f'\n\nIt is recommended that you run spy.upgrade() or issue the following PIP command to '
+                    f'install a compatible version of the seeq module:\n')
+        message += generate_pip_upgrade_command(session)
 
         if allow_version_mismatch or session.options.allow_version_mismatch:
             status.warn(message)
@@ -814,9 +777,68 @@ def validate_seeq_server_version(session: Session, status: Status, allow_version
     return seeq_server_major, seeq_server_minor, seeq_server_patch
 
 
-def generate_seeq_module_pip_upgrade(version, exact_match_version=False):
-    matcher = '==' if exact_match_version else '~='
-    return f'pip install -U seeq{matcher}{version}'
+def generate_pip_upgrade_command(session: Session, version: Optional[str] = None, use_testpypi: bool = False):
+    sdk_module_major, sdk_module_minor, _ = get_sdk_module_version_tuple()
+    seeq_server_major, seeq_server_minor, seeq_server_patch = get_server_version_tuple(session)
+    compatible_module_folder = find_compatible_module(session)
+    # The old versioning scheme is like 0.49.3 whereas the new scheme is like 50.1.8
+    # See https://seeq.atlassian.net/wiki/spaces/SQ/pages/947225963/Seeq+Versioning+Simplification
+    using_old_versioning_scheme = (seeq_server_major == 0)
+    first_command = None
+    if using_old_versioning_scheme:
+        install_compatible_sdk = (sdk_module_major != seeq_server_major or sdk_module_minor != seeq_server_minor)
+        compatible_sdk_version_specifier = f'{seeq_server_major}.{seeq_server_minor}.{seeq_server_patch}'
+    else:
+        install_compatible_sdk = (sdk_module_major != seeq_server_major)
+        compatible_sdk_version_specifier = f'{seeq_server_major}.{seeq_server_minor}'
+    repository_arg = ' --index-url https://test.pypi.org/simple/' if use_testpypi else ''
+
+    def _install_compatible_sdk():
+        if not install_compatible_sdk:
+            return None
+
+        if compatible_module_folder is not None:
+            return 'pip uninstall -y seeq'
+        else:
+            return f'pip install -U{repository_arg} seeq~={compatible_sdk_version_specifier}'
+
+    if version is not None:
+        if 'r' in version.lower():
+            version = re.sub(pattern='r', repl='', string=version, flags=re.IGNORECASE)
+
+        match = re.match(r'^(\d+)\..*', version)
+        if not match:
+            raise SPyValueError(f'version argument "{version}" is not a full version (e.g. 221.13 or 58.0.2.184.12)')
+
+        version_major = int(match.group(1))
+        if version_major < SEEQ_SERVER_VERSION_WHERE_SPY_IS_IN_ITS_OWN_PACKAGE:
+            # We're going to the old single-package scheme, where seeq and spy are in the same package and the
+            # versioning is something like 58.0.2.184.12. If the currently-installed sdk module is R60 or later, we must
+            # uninstall the seeq-spy package so that, when the older seeq package (which includes spy directly) is
+            # installed, pip doesn't think that seeq-spy is still installed as well.
+            if sdk_module_major >= SEEQ_SERVER_VERSION_WHERE_SPY_IS_IN_ITS_OWN_PACKAGE:
+                first_command = 'pip uninstall -y seeq-spy'
+
+            second_command = f'pip install -U{repository_arg} seeq=={version}'
+        else:
+            # We're going to the new seeq-spy package scheme.
+            if seeq_server_major < SEEQ_SERVER_VERSION_WHERE_SPY_IS_IN_ITS_OWN_PACKAGE:
+                raise SPyValueError(f'version argument "{version}" is incompatible with Seeq Server version '
+                                    f'{session.server_version}')
+
+            first_command = _install_compatible_sdk()
+            second_command = f'pip install -U{repository_arg} seeq-spy=={version}'
+    else:
+        if seeq_server_major >= SEEQ_SERVER_VERSION_WHERE_SPY_IS_IN_ITS_OWN_PACKAGE:
+            first_command = _install_compatible_sdk()
+            second_command = f'pip install -U{repository_arg} seeq-spy'
+        else:
+            if sdk_module_major >= SEEQ_SERVER_VERSION_WHERE_SPY_IS_IN_ITS_OWN_PACKAGE:
+                first_command = 'pip uninstall -y seeq-spy'
+            second_command = f'pip install -U{repository_arg} seeq~={compatible_sdk_version_specifier}'
+    pip_commands = [second_command] if first_command is None else [first_command, second_command]
+    pip_command = ' && '.join(pip_commands)
+    return pip_command
 
 
 def find_compatible_module(session: Session):
@@ -851,10 +873,10 @@ def validate_data_lab_license(session: Session):
                 return
 
             raise SPyRuntimeError(f'Seeq Data Lab license is "{additional_feature.validity}", could not log in. '
-                                  f'Contact your administrator or speak with Seeq Support at support@seeq.com.')
+                                  f'Contact your administrator or log a support ticket via https://support.seeq.com.')
 
     raise SPyRuntimeError('Seeq Data Lab is not licensed for this server, could not log in. Contact your administrator '
-                          'or speak with Seeq Support at support@seeq.com.')
+                          'or log a support ticket via https://support.seeq.com.')
 
 
 def validate_login(session: Session, status: Status):
@@ -864,13 +886,16 @@ def validate_login(session: Session, status: Status):
     validate_seeq_server_version(session, status)
 
 
-def parse_content_datetime_with_timezone(session: Session, dt: object, timestamp_units: str = 'ms',
+def parse_content_datetime_with_timezone(session: Optional[Session], dt: object, timestamp_units: str = 'ms',
                                          timezone: Optional[object] = None) -> pd.Timestamp:
-    if timezone is None:
+    if timezone is None and session is not None:
         timezone = get_user_timezone(session)
 
     if isinstance(dt, pd.Timestamp):
         if not pd.isna(dt) and dt.tz is None:
+            if timezone is None:
+                raise SPyValueError(f'Date/time object {dt} has no timezone and no default timezone was specified')
+
             dt = dt.tz_localize(timezone)
         return dt
 
@@ -881,13 +906,25 @@ def parse_content_datetime_with_timezone(session: Session, dt: object, timestamp
     if not isinstance(dt, str):
         raise SPyValueError(f'Date/time object {dt} of type "{type(dt)}" not recognized')
 
-    match = re.fullmatch(r'^([^\[\]]+)(\[(.+)])?$', dt)
-    if not match or not match.group(3):
+    match = re.fullmatch(r'^(\d{4}-\d{2}-\d{2}(?:T\d{2}:\d{2}:\d{2}(?:\.\d+)?)?)\[?(.*?)]?$', dt)
+
+    if match is None:
+        raise SPyValueError(f'Date/time object {dt} not recognized. It needs to be in ISO 8601 format,'
+                            f'e.g. 2020-01-01T00:00:00Z')
+
+    tz_part = match.group(2)
+    if tz_part == 'Z':
+        tz_part = 'UTC'
+
+    if not match or not tz_part:
+        if timezone is None:
+            raise SPyValueError(f'Date/time object {dt} has no timezone and no default timezone was specified')
+
         # noinspection PyTypeChecker
         return pd.Timestamp(dt, tz=timezone)
 
     datetime_part = match.group(1)
-    tz_part = match.group(3)
+
     return pd.Timestamp(datetime_part, tz=tz_part)
 
 
@@ -902,14 +939,8 @@ def get_fallback_timezone(session: Session):
 
 
 def parse_input_datetime(session: Session,
-                         input_datetime: Union[pd.Timestamp,
-                                               datetime.date,
-                                               str,
-                                               int,
-                                               float],
-                         timezone: Optional[Union[str,
-                                                  pytz.BaseTzInfo,
-                                                  tz.tzoffset]] = None) -> pd.Timestamp:
+                         input_datetime: Union[pd.Timestamp, datetime.date, str, int, float],
+                         timezone: Optional[Union[str, pytz.BaseTzInfo, tz.tzoffset]] = None) -> pd.Timestamp:
     """
     Takes a datetime and optionally a timezone, and returns a pd.Timestamp that is
     timezone-aware, by localizing naive datetimes to specified timezone if provided, else
@@ -946,9 +977,10 @@ def parse_input_datetime(session: Session,
     return input_datetime
 
 
-def validate_start_and_end(session: Session,
-                           start: Union[pd.Timestamp, datetime.date, str, int, float],
-                           end: Union[pd.Timestamp, datetime.date, str, int, float]):
+def validate_start_and_end(
+        session: Session,
+        start: Union[pd.Timestamp, datetime.date, str, int, float],
+        end: Union[pd.Timestamp, datetime.date, str, int, float]) -> Tuple[pd.Timestamp, pd.Timestamp]:
     """
     Takes a pair of start and end times, either of which could be None, and uses some default logic
     to produce a "cleaned up" start and end. For example, if start is after end, then end is reset to
@@ -1037,3 +1069,35 @@ def __getattr__(name):
     else:
         # fallback to default module attribute access
         raise AttributeError
+
+
+@dataclass
+class RequestOrigin:
+    label: Optional[str]
+    url: Optional[str]
+
+
+def _retrieve_default_request_origin(session: Session) -> Optional[RequestOrigin]:
+    if not ((_datalab.is_datalab() or _datalab.is_executor()) and not _datalab.is_datalab_api()):
+        return None
+
+    # noinspection PyBroadException
+    try:
+        if _datalab.is_executor():
+            project_name = os.environ.get('SEEQ_PROJECT_NAME', '')
+            file_path = os.environ.get('SEEQ_SDL_FILE_PATH', '')
+            project_url = _datalab.get_data_lab_project_url(use_private_url=False)
+            notebook_url = f"{project_url}/notebooks/{file_path}"
+            request_origin_label = f"[Scheduled] {project_name} - {file_path}"
+            request_origin_url = quote(urlparse(notebook_url).path)
+        else:
+            notebook_url = _datalab.get_notebook_url(session)
+            request_origin_url = quote(urlparse(notebook_url).path)
+
+            project_name = _datalab.get_data_lab_project_name(session)
+            file_path = notebook_url.split('notebooks/', 1)[1]
+            request_origin_label = project_name + " - " + file_path
+
+        return RequestOrigin(label=request_origin_label, url=request_origin_url)
+    except BaseException:
+        return None
