@@ -4,11 +4,12 @@ import json
 import os
 import re
 from dataclasses import dataclass
-from typing import Callable, Dict, Optional, List, Tuple, Set
+from typing import Callable, Dict, Optional, List, Tuple, Set, Union
 
 import numpy as np
 import pandas as pd
 
+from seeq import spy
 from seeq.base.seeq_names import SeeqNames
 from seeq.sdk import *
 from seeq.spy import _common, _login
@@ -112,7 +113,7 @@ def push(session: Session, metadata, workbook_context: WorkbookContext, datasour
     metadata_df.reset_index(inplace=True)
 
     if 'Type' not in metadata_df.columns:
-        metadata_df['Type'] = pd.Series(np.nan, dtype='object')
+        metadata_df['Type'] = pd.Series(np.nan, dtype=object)
 
     push_results = PushResults(metadata_df)
 
@@ -128,12 +129,18 @@ def push(session: Session, metadata, workbook_context: WorkbookContext, datasour
             if status.errors == 'raise':
                 raise
 
+    if (spy.utils.is_sdk_module_version_at_least(61, 1, 4) and not spy.utils.is_sdk_module_version_at_least(62)) or \
+            spy.utils.is_sdk_module_version_at_least(62, 0, 1):
+        put_scalars_input = PutScalarsInputV1(scalars=list(), force_calculated_scalars=True)
+    else:
+        put_scalars_input = PutScalarsInputV1(scalars=list())
+
     push_context = PushContext(
         reified_assets=set(),
         roots=dict(),
         batch_size=session.options.metadata_push_batch_size,
         put_signals_input=PutSignalsInputV1(signals=list()),
-        put_scalars_input=PutScalarsInputV1(scalars=list()),
+        put_scalars_input=put_scalars_input,
         condition_batch_input=ConditionBatchInputV1(conditions=list()),
         threshold_metric_inputs=list(),
         display_template_inputs=list(),
@@ -237,6 +244,10 @@ def push(session: Session, metadata, workbook_context: WorkbookContext, datasour
                   Status.SUCCESS)
 
     if state_file is not None:
+        state_file_folder = os.path.dirname(state_file)
+        if len(state_file_folder) != 0:
+            os.makedirs(state_file_folder, exist_ok=True)
+
         # Specifically use protocol 4 so that the file is portable back to Python 3.4
         results_df.to_pickle(state_file, protocol=4)
 
@@ -296,6 +307,21 @@ def _process_push_row(session: Session, status: Status, push_context: PushContex
     if _common.present(row_dict, 'Push Result'):
         return
 
+    def _row_datasource_matches_context_datasource() -> bool:
+        # We can't update tree relationships using the batch endpoint across multiple datasources. Don't bother with
+        # Asset updates if this item appears to be in a datasource that doesn't match the context. CRAB-37889
+        if _common.get(row_dict, 'Reference') is True:
+            return True
+        elif _common.present(row_dict, 'Datasource Class') and _common.present(row_dict, 'Datasource ID'):
+            return (_common.get(row_dict, 'Datasource Class') == push_context.datasource_output.datasource_class
+                    and _common.get(row_dict, 'Datasource ID') == push_context.datasource_output.datasource_id)
+        elif _common.present(row_dict, 'Datasource Name'):
+            return _common.get(row_dict, 'Datasource Name') == push_context.datasource_output.name
+        else:
+            return True
+
+    datasource_matches = _row_datasource_matches_context_datasource()
+
     scoped_data_id = get_scoped_data_id(row_dict, push_context.workbook_context.workbook_id)
     if not _common.present(row_dict, 'Datasource Class'):
         row_dict['Datasource Class'] = push_context.datasource_output.datasource_class
@@ -305,7 +331,7 @@ def _process_push_row(session: Session, status: Status, push_context: PushContex
 
     path = determine_path(row_dict)
     parent_data_id = None
-    if path:
+    if path and datasource_matches:
         parent_data_id = _reify_path(session, status, push_context, path)
 
     def _set_properties_by_id():
@@ -490,9 +516,9 @@ def _process_push_row(session: Session, status: Status, push_context: PushContex
                     threshold_metric_input.bounding_condition, push_context.workbook_context.workbook_id,
                     push_context.push_results)
 
-            threshold_metric_input.thresholds = _convert_thresholds_dict_to_input(
+            threshold_metric_input.thresholds = _convert_thresholds_to_input(
                 threshold_metric_input.thresholds, push_context.workbook_context.workbook_id,
-                push_context.push_results)
+                push_context.push_results, row_dict)
 
             if _common.present(row_dict, 'Statistic'):
                 threshold_metric_input.aggregation_function = _common.statistic_to_aggregation_function(
@@ -646,7 +672,7 @@ def _get_push_directives(row_dict):
     return push_directives
 
 
-TYPE_SIMPLIFIER_REGEX = re.compile(r'(Stored|Calculated|Threshold)')
+TYPE_SIMPLIFIER_REGEX = re.compile(r'(Stored|Calculated|Threshold|Literal)')
 
 
 def _maybe_skip_item(session: Session, push_context: PushContext, scoped_data_id: str, row_dict: dict,
@@ -766,6 +792,10 @@ def _set_item_properties(session: Session, row_dict, sync_token):
 
     excluded_properties = [p for p in IGNORED_PROPERTIES if p not in do_not_exclude]
 
+    if _common.get(row_dict, 'Type') == 'LiteralScalar':
+        # LiteralScalar doesn't allow updating value uom directly since it's updated when setting value
+        excluded_properties.append(SeeqNames.Properties.value_uom)
+
     props = [
         ScalarPropertyV1(name=_name, value=_value) for _name, _value in row_dict.items()
         if _name not in excluded_properties and (isinstance(_value, list) or not pd.isna(_value))
@@ -825,7 +855,13 @@ def determine_path(d):
     return _common.path_list_to_string(path)
 
 
-def get_scoped_data_id(d, workbook_id):
+def get_scoped_data_id(d: dict, workbook_id: Optional[str]) -> str:
+    """
+    :param d: The dictionary representing this item.
+    :param workbook_id: The ID of the workbook that the item is scoped to. None or EMPTY_GUID represents global.
+    :return: The generated Data ID representing this item. This is generally in the form of
+             "[workbook_id] {item_type} path >> to >> item".
+    """
     path = determine_path(d)
 
     if not _common.present(d, 'Data ID'):
@@ -842,7 +878,7 @@ def get_scoped_data_id(d, workbook_id):
 
         guid = workbook_id if workbook_id else EMPTY_GUID
 
-        _type = d['Type'].replace('Stored', '').replace('Calculated', '')
+        _type = d['Type'].replace('Stored', '').replace('Calculated', '').replace('Literal', '')
         if 'Metric' in _type:
             _type = 'ThresholdMetric'
 
@@ -1184,6 +1220,7 @@ def build_reference(session: Session, definition):
         'CalculatedSignal': _build_reference_signal,
         'StoredCondition': _build_reference_condition,
         'CalculatedCondition': _build_reference_condition,
+        'LiteralScalar': _build_reference_scalar,
         'CalculatedScalar': _build_reference_scalar
     }[definition['Type']](session, definition)
 
@@ -1453,8 +1490,12 @@ def _flush(session: Session, status: Status, push_context: PushContext):
     _post_batch_async(session, lambda: trees_api.batch_move_nodes_to_parents(body=tree_body), tree_body.relationships,
                       status, push_context.push_results)
 
+    if (spy.utils.is_sdk_module_version_at_least(61, 1, 4) and not spy.utils.is_sdk_module_version_at_least(62)) or \
+            spy.utils.is_sdk_module_version_at_least(62, 0, 1):
+        push_context.put_scalars_input = PutScalarsInputV1(scalars=list(), force_calculated_scalars=True)
+    else:
+        push_context.put_scalars_input = PutScalarsInputV1(scalars=list())
     push_context.put_signals_input = PutSignalsInputV1(signals=list())
-    push_context.put_scalars_input = PutScalarsInputV1(scalars=list())
     push_context.condition_batch_input = ConditionBatchInputV1(conditions=list())
     push_context.asset_batch_input = AssetBatchInputV1(assets=list())
     push_context.tree_batch_input = AssetTreeBatchInputV1(relationships=list(),
@@ -1509,30 +1550,6 @@ def _polyfill_post_batch(session: Session, item_inputs, create_item_endpoint, ac
     item_inputs.clear()
 
 
-def _convert_thresholds_dict_to_input(thresholds_dict, workbook_id, push_results: PushResults):
-    """
-    Convert a dictionary with keys threshold levels and values of either scalars or metadata to a list of strings
-    with level=value/ID of the threshold.
-
-    :param thresholds_dict: A dictionary with keys of threshold levels and values of either number of metadata
-    dataframes
-    :return:  A list of strings 'level=value' or 'level=ID'
-    """
-    thresholds_list = list()
-    if thresholds_dict is not None:
-        if not isinstance(thresholds_dict, dict):
-            raise SPyTypeError(f'"Thresholds" value for Metric should be dict, but instead is type '
-                               f'{type(thresholds_dict).__name__} with value:\n{thresholds_dict}')
-
-        for k, v in thresholds_dict.items():
-            if isinstance(v, pd.DataFrame) or isinstance(v, dict):
-                thresholds_list.append(
-                    f'{k}={_item_id_from_parameter_value(v, workbook_id, push_results)}')
-            else:
-                thresholds_list.append(f'{k}={v}')
-    return thresholds_list
-
-
 def _add_no_dupe(lst, obj, attr='data_id', overwrite=False):
     for i in range(0, len(lst)):
         o = lst[i]
@@ -1828,6 +1845,79 @@ def formula_parameters_dict_from_threshold_metric(session: Session, item_id):
     return formula_parameters
 
 
+def _get_threshold_level(threshold_entry: dict) -> str:
+    """
+    :param threshold_entry: A dictionary of a single Threshold, like Appserver outputs, which includes the
+    Priority (Name + Level + optional Color) and a Value (either a Scalar or ID).
+    :return: The Level as a string. Includes the numeric Level value plus the optional Color.
+    """
+    return '{}{}'.format(threshold_entry['Priority']['Level'],
+                         # A custom color can be specified for the threshold by appending it to the priority
+                         # number as a hex code. Example: '1#00ff00=10'
+                         threshold_entry['Priority']['Color'] if 'Color' in threshold_entry['Priority'] else '')
+
+
+def _get_threshold_value(threshold_entry: dict, item_map: Optional[dict] = None, current_item=None) -> Optional[str]:
+    """
+    :param threshold_entry: A dictionary of a single Threshold, like Appserver outputs, which includes the
+    Priority (Name + Level + optional Color) and a Value (either a Scalar or ID).
+    :param item_map: The optional map of old:new items being mapped to translate the specified ID.
+    :param current_item: The current item being pushed, just used for outputting errors.
+    :return: The value (scalar value or ID) of a single threshold entry.
+    """
+    _threshold_value = _common.get(threshold_entry, 'Value')
+    if _threshold_value is not None:
+        if isinstance(_threshold_value, dict):
+            return str_from_scalar_value_dict(_threshold_value)
+        else:
+            return _threshold_value
+    elif _common.present(threshold_entry, 'Item ID'):
+        if item_map is None:
+            return threshold_entry['Item ID']
+        if threshold_entry['Item ID'] not in item_map:
+            raise SPyDependencyNotFound(
+                f'{current_item} Threshold Metric threshold {threshold_entry["Item ID"]}) not found')
+
+
+def _convert_thresholds_to_input(thresholds_obj: Union[dict[str, any], list[Union[str, dict]]],
+                                 workbook_id: str, push_results: PushResults, current_item) -> list[str]:
+    """
+    Convert a dictionary with keys threshold levels and values of either scalars or metadata to a list of strings
+    with level=value/ID of the threshold.
+
+    :param thresholds_obj: Either a dictionary with keys of threshold levels and values of either number of metadata
+    dataframes or a list of threshold dictionaries like what Appserver returns.
+    :param workbook_id: The ID of the workbook being pushed. Used for looking up mapped item IDs.
+    :param push_results: The total push results. Used for looking up mapped item IDs.
+    :param current_item: The current item being pushed, just used for outputting errors.
+    :return: A list of strings 'level[#optional_color]=value' or 'level[#optional_color]=ID'
+    """
+    thresholds_list = list()
+    if thresholds_obj is None:
+        return thresholds_list
+
+    if isinstance(thresholds_obj, dict):
+        thresholds_dict = thresholds_obj
+    elif isinstance(thresholds_obj, list):
+        thresholds_dict = dict()
+        for threshold_entry in thresholds_obj:
+            level = _get_threshold_level(threshold_entry)
+            value = _get_threshold_value(threshold_entry, None, current_item)
+            if level and value:
+                thresholds_dict[level] = value
+    else:
+        raise SPyTypeError(f'{current_item} Threshold Metric "Thresholds" value should be dict or list, '
+                           f'but instead is type {type(thresholds_obj).__name__} with value:\n{thresholds_obj}')
+
+    for k, v in thresholds_dict.items():
+        threshold = f'{k}={_item_id_from_parameter_value(v, workbook_id, push_results)}' \
+            if isinstance(v, pd.DataFrame) or isinstance(v, dict) \
+            else f'{k}={v}'
+        thresholds_list.append(threshold)
+
+    return thresholds_list
+
+
 def threshold_metric_input_from_formula_parameters(parameters, *, item_object=None,
                                                    item_map=None) -> ThresholdMetricInputV1:
     new_item = ThresholdMetricInputV1()
@@ -1848,29 +1938,16 @@ def threshold_metric_input_from_formula_parameters(parameters, *, item_object=No
 
             setattr(new_item, _attr, item_map[parameters[_key].upper()])
 
-    def _add_thresholds(_list, _key):
-        if not _common.present(parameters, _key):
-            return
-
-        for threshold_dict in parameters[_key]:
-            threshold_value = _common.get(threshold_dict, 'Value')
-            if threshold_value is not None:
-                if isinstance(threshold_value, dict):
-                    _list.append('%s=%s' % (threshold_dict['Priority']['Level'],
-                                            str_from_scalar_value_dict(threshold_value)))
-                else:
-                    _list.append('%s=%s' % (threshold_dict['Priority']['Level'], threshold_value))
-            elif _common.present(threshold_dict, 'Item ID'):
-                if item_map is None:
-                    _list.append('%s=%s' % (threshold_dict['Priority']['Level'], threshold_dict['Item ID']))
-                    return
-
-                if threshold_dict['Item ID'] not in item_map:
-                    raise SPyDependencyNotFound(
-                        f'{item_object} Threshold Metric threshold {threshold_dict["Item ID"]}) not found')
-
-                _list.append('%s=%s' % (threshold_dict['Priority']['Level'],
-                                        item_map[threshold_dict['Item ID']]))
+    def _get_thresholds() -> list[str]:
+        _thresholds_list = list()
+        if not _common.present(parameters, 'Thresholds'):
+            return _thresholds_list
+        for threshold_dict in parameters['Thresholds']:
+            level = _get_threshold_level(threshold_dict)
+            value = _get_threshold_value(threshold_dict, item_map, item_object)
+            if level and value:
+                _thresholds_list.append(f'{level}={value}')
+        return _thresholds_list
 
     new_item.aggregation_function = _common.get(parameters, 'Aggregation Function')
     _add_mapped_item('bounding_condition', 'Bounding Condition')
@@ -1878,8 +1955,7 @@ def threshold_metric_input_from_formula_parameters(parameters, *, item_object=No
     _add_scalar_value('duration', 'Duration')
     _add_mapped_item('measured_item', 'Measured Item')
     _add_scalar_value('period', 'Period')
-    new_item.thresholds = list()
-    _add_thresholds(new_item.thresholds, 'Thresholds')
+    new_item.thresholds = _get_thresholds()
 
     return new_item
 
@@ -1896,10 +1972,11 @@ RESERVED_SPY_COLUMN_NAMES = [
                                 'Formula Parameters', 'Capsule Is Uncertain', 'Capsule Property Units',
                                 'Override Number Format', 'Condition',
                                 'Permissions Inheritance Disabled', 'Access Control',
-                                'Roll Up Statistic', 'Roll Up Parameters',
+                                'Roll Up Statistic', 'Roll Up Parameters', 'Old Asset Format',
                                 'Template ID', 'Swap Out Asset ID', 'Swap In Asset ID', 'Source Workstep ID'
                             ] + RESERVED_SPY_STATUS_COLUMN_NAMES
 
+# Properties that will not be supplied via the "additionalProperties" field.
 # This must be kept in sync with RESERVED_ITEM_PROPERTIES in StoredItemOutput.java
 RESERVED_ITEM_PROPERTIES = [
     SeeqNames.Properties.guid,
@@ -1908,6 +1985,7 @@ RESERVED_ITEM_PROPERTIES = [
     SeeqNames.Properties.data_id,
     SeeqNames.Properties.name,
     SeeqNames.Properties.description,
+    SeeqNames.Properties.metadata_properties,
     SeeqNames.Properties.interpolation_method,
     SeeqNames.Properties.maximum_duration,
     SeeqNames.Properties.maximum_interpolation,
@@ -1922,16 +2000,22 @@ RESERVED_ITEM_PROPERTIES = [
     SeeqNames.Properties.formula_parameters,
     SeeqNames.Properties.u_i_config,
     SeeqNames.Properties.sync_token,
+    SeeqNames.Properties.permission_inheritance_disabled,
     SeeqNames.Properties.permissions_from_datasource,
     SeeqNames.Properties.source_security_string,
     SeeqNames.Properties.security_string,
-    SeeqNames.Properties.cache_id
+    SeeqNames.Properties.cache_id,
+    SeeqNames.Properties.created_at,
+    SeeqNames.Properties.created_by,
+    SeeqNames.Properties.updated_at,
+    SeeqNames.Properties.last_viewed_at
 ]
 
 ORIGINAL_INDEX_COLUMN = '__Original Index__'
 
 ADDITIONAL_RESERVED_PROPERTIES = [SeeqNames.Properties.scoped_to, SeeqNames.Properties.metadata_properties,
                                   SeeqNames.Properties.storage_location, SeeqNames.Properties.stored_in_seeq,
+                                  SeeqNames.Properties.aggregation_function,
                                   'Cached By Service', 'Data Version Check']
 
 IGNORED_PROPERTIES = (RESERVED_SPY_COLUMN_NAMES + RESERVED_ITEM_PROPERTIES + ADDITIONAL_RESERVED_PROPERTIES +
