@@ -16,12 +16,13 @@ from seeq.spy import _login
 from seeq.spy._errors import *
 from seeq.spy._redaction import request_safely, safely
 from seeq.spy._session import Session
-from seeq.spy._status import Status
+from seeq.spy._status import Status, Usage
 
 ENUM_REGEX = r'ENUM{{(\d+)\|(.+?)}}'
 ENUM_PATTERN = re.compile(ENUM_REGEX)
 
 
+@Status.handle_keyboard_interrupt()
 def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_by=None, shape='auto',
          capsule_properties=None, tz_convert=None, calculation=None, bounding_values=False,
          invalid_values_as=np.nan, enums_as='string', errors=None, quiet=None, status: Status = None,
@@ -38,7 +39,7 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
         spy.search(). Alternatively, you can supply URL of a Seeq Workbench
         worksheet as a str.
 
-    start : {str, pd.Timestamp}
+    start : {str, pd.Timestamp}, optional
         The starting time for which to pull data. This argument must be a
         string that pandas.to_datetime() can parse, or a pandas.Timestamp.
         If not provided, 'start' will default to 'end' minus 1 hour. Note
@@ -48,7 +49,7 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
         is a str, 'start' will default to the start of the display range
         in Seeq Trend View.
 
-    end : {str, pd.Timestamp}
+    end : {str, pd.Timestamp}, optional
         The end time for which to pull data. This argument must be a string
         that pandas.to_datetime() can parse, or a pandas.Timestamp.
         If not provided, 'end' will default to now. Note that Seeq will
@@ -74,12 +75,12 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
         creation. To change the interpolation type for a given signal, change the
         signal's interpolation or use the appropriate 'calculation' argument.
 
-    header : str default '__auto__'
+    header : str, default '__auto__'
         The metadata property to use as the header of each column. Common
         values would be 'ID' or 'Name'. '__auto__' concatenates Path and Name
         if they are present.
 
-    group_by : {str, list(str)}
+    group_by : {str, list(str)}, optional
         The name of a column or list of columns for which to group by. Often
         necessary when pulling data across assets: When you want header='Name',
         you typically need group_by=['Path', 'Asset']
@@ -90,19 +91,19 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
         rows if no signals are present. 'samples' or 'capsules' forces the
         output to the former or the latter, if possible.
 
-    capsule_properties : list(str)
+    capsule_properties : list(str), optional
         A list of capsule properties to retrieve when shape='capsules'.
         By default, if no signals are present in the items DataFrame, then all
         properties found on a capsule are automatically returned (because
         the nature of the query allows them to be returned "for free").
         Otherwise, you must provide a list of names of properties to retrieve.
 
-    tz_convert : {str, datetime.tzinfo}
+    tz_convert : {str, datetime.tzinfo}, optional
         The time zone in which to return all timestamps. If the time zone
         string is not recognized, the list of supported time zone strings will
         be returned in the exception text.
 
-    calculation : {str, pandas.Series, pandas.DataFrame}
+    calculation : {str, pandas.Series, pandas.DataFrame}, optional
         When applying a calculation across assets, the 'calculation' argument
         must be a one-row DataFrame (or a Series) and the 'items' argument must
         be full of assets. When applying a calculation to a signal/condition/
@@ -142,7 +143,7 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
     quiet : bool, default False
         If True, suppresses progress output. Note that when status is
         provided, the quiet setting of the Status object that is passed
-        in takes precedent.
+        in takes precedence.
 
     status : spy.Status, optional
         If specified, the supplied Status object will be updated as the command
@@ -328,7 +329,7 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
         calculation = calculation.iloc[0]
 
     if isinstance(items, pd.Series):
-        items = pd.DataFrame().append(items)
+        items = pd.DataFrame([items])
 
     if shape not in ['auto', 'capsules', 'samples']:
         raise SPyValueError("shape must be one of 'auto', 'capsules', 'samples'")
@@ -356,6 +357,7 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
     status.df['Time'] = 0
     status.df['Count'] = 0
     status.df['Pages'] = 0
+    status.df['Data Processed'] = Usage()
     status.df['Result'] = 'Pulling'
 
     query_df = items  # type: pd.DataFrame
@@ -495,10 +497,12 @@ def pull(items, *, start=None, end=None, grid='15min', header='__auto__', group_
             # When the shape of the output is capsules, it's effectively just a long
             # DataFrame with all of the results appended one after the other. The user can
             # then choose to sort by one of the columns if they would like.
-            output.df = output.df.append(result_list, ignore_index=True)
+            result_list.insert(0, output.df)
+            output.df = pd.concat(result_list, ignore_index=True)
         else:
             if group_by:
-                output.df = output.df.append(result_list)
+                result_list.insert(0, output.df)
+                output.df = pd.concat(result_list)
                 output.df = output.df.groupby(['Timestamp'] + group_by).first()
             else:
                 # noinspection PyTypeChecker
@@ -540,7 +544,7 @@ def _process_query_row(session: Session, at_least_one_signal, calculation, shape
         status.df.at[row_index, 'Result'] = 'No "ID" column - skipping'
         return None
 
-    item_id, item_name, item_type = _get_item_details(session, header, row)
+    item_id, item_name, item_type = _get_item_details(session, header, row, query_df)
 
     calculation_to_use = calculation
     if item_type == 'Asset':
@@ -554,7 +558,7 @@ def _process_query_row(session: Session, at_least_one_signal, calculation, shape
         def _request_swap_items(_item_id, _item_type, _calculation_to_use):
             swap_input = SwapInputV1()
             swap_input.swap_in = _item_id
-            calc_item_id, _, _item_type = _get_item_details(session, header, calculation)
+            calc_item_id, _, _item_type = _get_item_details(session, header, calculation, query_df)
 
             item_dependency_output = items_api.get_formula_dependencies(id=calc_item_id)  # type: ItemDependencyOutputV1
 
@@ -640,15 +644,17 @@ def _pull_condition(session: Session, shape, capsule_properties, calculation_to_
     page_count = 0
     indices_to_update = {row_index}
 
-    def _update_status_rows(_message, _capsule_count, _page_count):
+    def _update_status_rows(_message, _capsule_count, _page_count, _usage):
         for _row_index_to_update in indices_to_update:
             status.send_update(_row_index_to_update, {
                 'Result': _message,
                 'Count': _capsule_count,
                 'Pages': _page_count,
-                'Time': _common.timer_elapsed(timer)
+                'Time': _common.timer_elapsed(timer),
+                'Data Processed': _usage
             })
 
+    usage = Usage()
     while True:
         # When we want capsule summary statistics, fetch as a capsule table
         if shape == 'capsules' and at_least_one_signal:
@@ -656,17 +662,19 @@ def _pull_condition(session: Session, shape, capsule_properties, calculation_to_
                 raise SPyRuntimeError("If shape='capsules' and at least one signal is present, calculation "
                                       "argument cannot be supplied")
 
-            this_capsule_count, next_start, result_df = \
+            this_capsule_count, next_start, result_df, http_headers = \
                 _pull_condition_as_a_table(session, capsule_properties, current_start, query_df, item_id, item_name,
                                            header, offset, pd_end, tz, result_df, column_names, indices_to_update)
         else:
-            this_capsule_count, next_start, result_df = _pull_condition_via_formula_api(
+            this_capsule_count, next_start, result_df, http_headers = _pull_condition_via_formula_api(
                 session, calculation_to_use, shape, capsule_properties, current_start,
                 index_to_use, item_id, item_name, offset, pd_end, tz, result_df, column_names)
 
         # Note that capsule_count here can diverge from the exact count in the output due to pagination
         capsule_count += this_capsule_count
 
+        usage.add(http_headers)
+
         page_count += 1
 
         if this_capsule_count < session.options.pull_page_size:
@@ -680,20 +688,21 @@ def _pull_condition(session: Session, shape, capsule_properties, calculation_to_
 
         current_start = next_start
 
-        _update_status_rows(f'Pulling {_common.convert_to_timestamp(current_start, tz)}', capsule_count, page_count)
+        _update_status_rows(f'Pulling {_common.convert_to_timestamp(current_start, tz)}', capsule_count, page_count,
+                            usage)
 
-    _update_status_rows(f'Success', capsule_count, page_count)
+    _update_status_rows(f'Success', capsule_count, page_count, usage)
 
     return RowResult(row_index, column_names, result_df)
 
 
 def _is_capsule_dupe(result_df, item_name, pd_capsule_start, pd_capsule_end):
     return 'Condition' in result_df and \
-           'Capsule Start' in result_df and \
-           'Capsule End' in result_df and \
-           len(result_df.loc[(result_df['Condition'] == item_name) &
-                             (result_df['Capsule Start'] == pd_capsule_start) &
-                             (result_df['Capsule End'] == pd_capsule_end)])
+        'Capsule Start' in result_df and \
+        'Capsule End' in result_df and \
+        len(result_df.loc[(result_df['Condition'] == item_name) &
+                          (result_df['Capsule Start'] == pd_capsule_start) &
+                          (result_df['Capsule End'] == pd_capsule_end)])
 
 
 def _pull_condition_via_formula_api(session: Session, calculation_to_use, shape, capsule_properties, current_start,
@@ -747,7 +756,7 @@ def _pull_condition_via_formula_api(session: Session, calculation_to_use, shape,
                 colname = '%s - %s' % (item_name, prop['name'])
                 if colname not in columns:
                     # Here we start with a NaN-filled series, since we're populating property values (not 1s and 0s).
-                    columns[colname] = pd.Series(np.nan, index_to_use)
+                    columns[colname] = pd.Series(np.nan, index_to_use, dtype=object)
 
                 # Note here that overlapping capsules with different properties will result in "last one wins"
                 columns[colname].loc[(columns[colname].index >= pd_capsule_start) &
@@ -761,7 +770,7 @@ def _pull_condition_via_formula_api(session: Session, calculation_to_use, shape,
         # In this case, we're creating a more straightforward table where each capsule is a row, complete with item
         # properties.
 
-        capsule_df_rows = list()
+        capsule_dict_list = list()
         if len(capsules_output['capsules']) > 0:
             column_names[item_id] = ['Condition', 'Capsule Start', 'Capsule End', 'Capsule Is Uncertain']
 
@@ -791,14 +800,15 @@ def _pull_condition_via_formula_api(session: Session, calculation_to_use, shape,
                 if prop['name'] not in column_names[item_id]:
                     column_names[item_id].append(prop['name'])
 
-            capsule_df_rows.append(capsule_dict)
+            capsule_dict_list.append(capsule_dict)
 
             if not pd.isna(capsule_start) and capsule_start > next_start:
                 next_start = capsule_start
 
-        result_df = result_df.append(capsule_df_rows) if len(result_df) != 0 else pd.DataFrame(capsule_df_rows)
+        result_df = (pd.concat([result_df, pd.DataFrame(capsule_dict_list)])
+                     if len(result_df) != 0 else pd.DataFrame(capsule_dict_list))
 
-    return len(capsules_output['capsules']), next_start, result_df
+    return len(capsules_output['capsules']), next_start, result_df, http_headers
 
 
 def _build_limit_fragment(offset, limit):
@@ -808,10 +818,10 @@ def _build_limit_fragment(offset, limit):
 
 
 def _build_stat_formula_fragment_and_update_parameters(session: Session, signals_df, header, parameters,
-                                                       indices_to_update):
+                                                       indices_to_update, query_df):
     signal_id_to_stats = defaultdict(list)
     for signal_index, signal_row in signals_df.iterrows():
-        signal_item_id, signal_item_name, signal_item_type = _get_item_details(session, header, signal_row)
+        signal_item_id, signal_item_name, signal_item_type = _get_item_details(session, header, signal_row, query_df)
         statistic = signal_row['Statistic'] if 'Statistic' in signal_row else 'average'
 
         stat_formula = _common.statistic_to_aggregation_function(statistic, allow_condition_stats=False).split('(')[0]
@@ -860,7 +870,7 @@ def _pull_condition_as_a_table(session: Session, capsule_properties, current_sta
     property_column_fragment = f'capsuleTable({capsule}, CapsuleBoundary.Overlap, {group_segment}, $condition)'
     sort_fragment = ".sort('Capsule ID', 'inv, asc', 'Condition Id', 'asc', 'Capsule SortKey', 'asc')"
     stat_fragment, parameters, stat_headers = _build_stat_formula_fragment_and_update_parameters(
-        session, signals_df, header, parameters, indices_to_update)
+        session, signals_df, header, parameters, indices_to_update, query_df)
     formula = property_column_fragment + stat_fragment + sort_fragment + limit_fragment
     limit = session.options.pull_page_size
 
@@ -901,7 +911,7 @@ def _pull_condition_as_a_table(session: Session, capsule_properties, current_sta
             'Condition': item_name,
             'Capsule Start': pd_capsule_start,
             'Capsule End': pd_capsule_end,
-            'Capsule Is Uncertain': row[column_names_to_indexes['Original Uncertainty']]
+            'Capsule Is Uncertain': bool(row[column_names_to_indexes['Original Uncertainty']])
         }
 
         # capsuleSortKey divides the properties and the stat columns in returned table
@@ -916,10 +926,10 @@ def _pull_condition_as_a_table(session: Session, capsule_properties, current_sta
         if not pd.isna(capsule_start) and capsule_start > next_start:
             next_start = capsule_end
 
-    formula_result_df = result_df.copy().append(capsule_df_rows) if len(result_df) != 0 else pd.DataFrame(
-        capsule_df_rows)
+    formula_result_df = (pd.concat([result_df, pd.DataFrame(capsule_df_rows)])
+                         if len(result_df) != 0 else pd.DataFrame(capsule_df_rows))
 
-    return len(formula_table), next_start, formula_result_df
+    return len(formula_table), next_start, formula_result_df, http_headers
 
 
 def _pull_signal(session: Session, formula, parameters, item_id, item_name, pd_start, pd_end, tz,
@@ -933,6 +943,7 @@ def _pull_signal(session: Session, formula, parameters, item_id, item_name, pd_s
     current_start = pd_start
     last_key = 0
     page_count = 0
+    usage = Usage()
 
     while True:
         start_string = '%d ns' % current_start.value
@@ -950,6 +961,8 @@ def _pull_signal(session: Session, formula, parameters, item_id, item_name, pd_s
             _response_type='json'
         )
 
+        usage.add(http_headers)
+
         if not _common.present(formula_run_output, 'samples'):
             # noinspection PyStringFormat
             raise SPyRuntimeError('formula_run_output.samples is None.\n'
@@ -988,7 +1001,7 @@ def _pull_signal(session: Session, formula, parameters, item_id, item_name, pd_s
         time_index = _convert_column_timezone(pd.DatetimeIndex(timestamps), tz)
 
         new_series = pd.Series(values, index=time_index, dtype=object)
-        series = new_series if series is None else series.append(new_series)
+        series = new_series if series is None else pd.concat([series, new_series])
 
         page_count += 1
 
@@ -1010,7 +1023,8 @@ def _pull_signal(session: Session, formula, parameters, item_id, item_name, pd_s
                 'Result': f'Pulling: {current_start}',
                 'Count': len(series),
                 'Pages': page_count,
-                'Time': _common.timer_elapsed(timer)
+                'Time': _common.timer_elapsed(timer),
+                'Data Processed': usage
             })
 
     series = series.fillna(invalid_values_as)
@@ -1020,7 +1034,8 @@ def _pull_signal(session: Session, formula, parameters, item_id, item_name, pd_s
             'Result': 'Success',
             'Count': len(series),
             'Pages': page_count,
-            'Time': _common.timer_elapsed(timer)
+            'Time': _common.timer_elapsed(timer),
+            'Data Processed': usage
         })
 
     return RowResult(row_index, {item_id: [item_name]}, pd.DataFrame({item_name: series}))
@@ -1045,12 +1060,15 @@ def _pull_scalar(session: Session, formula, parameters, row_index, item_id, item
                  status: Status):
     formulas_api = FormulasApi(session.client)
     timer = _common.timer_start()
+    usage = Usage()
 
     formula_run_output, _, http_headers = formulas_api.run_formula_with_http_info(
-        formula=formula,
-        parameters=parameters)  # type: FormulaRunOutputV1
+        formula=formula, parameters=parameters)
 
-    status.send_update(row_index, {'Result': 'Success', 'Count': 1, 'Time': _common.timer_elapsed(timer)})
+    usage.add(http_headers)
+
+    status.send_update(row_index, {'Result': 'Success', 'Count': 1, 'Time': _common.timer_elapsed(timer),
+                                   'Data Processed': usage})
 
     if len(index_to_use) == 0:
         index_to_use = pd.Series([0])
@@ -1068,7 +1086,7 @@ class RowResult:
         self.result = result
 
 
-def _get_item_details(session: Session, header, row):
+def _get_item_details(session: Session, header, row, query_df):
     # This is a somewhat complex function that tries its best to pick a column header (item_name) for the output
     # DataFrame by either honoring the user's "header" argument or auto-picking something that makes sense.
 
@@ -1095,8 +1113,10 @@ def _get_item_details(session: Session, header, row):
 
         item_name = ''
         if header == '__auto__' and _common.present(row, 'Path'):
+            old_asset_format = (hasattr(query_df, 'spy') and hasattr(query_df.spy, 'old_asset_format') and
+                                query_df.spy.old_asset_format)
             item_name = _common.get(row, 'Path') + ' >> '
-            if _common.present(row, 'Asset'):
+            if (item_type != 'Asset' or old_asset_format) and _common.present(row, 'Asset'):
                 item_name += _common.get(row, 'Asset') + ' >> '
 
         if header in ['__auto__', 'Name']:
