@@ -9,6 +9,8 @@ from typing import Optional, Union
 
 import requests
 from bs4 import BeautifulSoup
+
+from seeq.base.seeq_names import SeeqNames
 from seeq.sdk import *
 from seeq.spy import _common
 from seeq.spy import _login
@@ -18,17 +20,19 @@ from seeq.spy._session import Session
 from seeq.spy._status import Status
 from seeq.spy.workbooks import _item, _render
 from seeq.spy.workbooks._content import Content, DateRange, AssetSelection
+from seeq.spy.workbooks._item import Item
 from seeq.spy.workbooks._item_map import ItemMap, OverrideItemMap
 from seeq.spy.workbooks._workstep import AnalysisWorkstep
 
 FIXUP_PREFIX = 'Annotation to fix up'
 
 
-class Annotation:
+class Annotation(Item):
     _html: str
     _images: dict
 
     def __init__(self, worksheet, annotation_type):
+        super().__init__()
         self.annotation_type = annotation_type
         self.worksheet = worksheet
         self._html = ''
@@ -53,11 +57,12 @@ class Annotation:
         self._set_html(new_item.html)
         self._images = new_item.images
 
-    def _find_image_references(self):
-        if not self.html:
+    @staticmethod
+    def _find_image_references(html):
+        if not html:
             return list()
 
-        matches = re.finditer(r'src="/api(/annotations/(.*?)/images/(.*?))"', self.html)
+        matches = re.finditer(r'src="/api(/annotations/(.*?)/images/(.*?))"', html)
         return [(match.group(1), match.group(2), match.group(3)) for match in matches]
 
     @property
@@ -141,7 +146,7 @@ class Annotation:
 
         return workstep_tuples
 
-    def pull(self, session: Session, *, include_images=True, status=None):
+    def pull(self, session: Session, *, include_images=True, status=None) -> Optional[AnnotationOutputV1]:
         session = Session.validate(session)
         self._images = dict()
         annotations_api = AnnotationsApi(session.client)
@@ -166,8 +171,15 @@ class Annotation:
         self.id = annotation_output.id
         self._set_html(annotation_output.document)
 
+        definition = safely(
+            lambda: Item._dict_from_item_output(Item._get_item_output(session, self.id)),
+            action_description=f'get properties of Annotation {self.id}',
+            status=status)
+
+        self._definition = definition
+
         if include_images:
-            image_references = self._find_image_references()
+            image_references = Annotation._find_image_references(self.html)
             for query_params, annotation_id, image_id in image_references:
                 if (annotation_id, image_id) in self.images:
                     continue
@@ -192,6 +204,7 @@ class Annotation:
         new_annotation.type = self.annotation_type
 
         annotations_api = AnnotationsApi(session.client)
+        items_api = ItemsApi(session.client)
         # Note: get_annotations() won't error if the user doesn't have access to the `annotates` IDs
         existing_annotations = annotations_api.get_annotations(
             annotates=[pushed_worksheet_id])  # type: AnnotationListOutputV1
@@ -226,16 +239,15 @@ class Annotation:
         # template
         images_map = dict()
 
-        # First add a prefix to existing image references in the HTML so that we can handle cases where
+        # We add a prefix to existing image references in the HTML so that we can handle cases where
         # the names that are picked by "POST /annotations/{id}/images" overlap with existing names and would
         # otherwise cause the find & replace that happens later to be non-deterministic
         image_prefix = 'ToBeMapped_'
-        html = re.sub(r'src="(/api/annotations/.*?/images/)(.*?)"', rf'src="\1{image_prefix}\2"', html)
 
         if push_images:
             item_map[f'Images for {self.id}'] = images_map
 
-            for _, annotation_id, image_id in self._find_image_references():
+            for _, annotation_id, image_id in Annotation._find_image_references(html):
                 api_client_url = session.get_api_url()
                 request_url = api_client_url + '/annotations/%s/images' % relevant_annotation.id
 
@@ -266,6 +278,8 @@ class Annotation:
         else:
             images_map = item_map[f'Images for {self.id}']
 
+        html = re.sub(r'src="(/api/annotations/.*?/images/)(.*?)"', rf'src="\1{image_prefix}\2"', html)
+
         bs = BeautifulSoup(html, features='html.parser')
         find_result = bs.find_all(['p', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'title'])
         name = 'Unnamed'
@@ -304,7 +318,8 @@ class Annotation:
                 continue
             new_interest = AnnotationInterestInputV1()
             new_interest.interest_id = interest_item.id
-            new_interest.detail_id = interest.capsule
+            if interest.capsule is not None:
+                new_interest.detail_id = interest.capsule.id
             new_annotation.interests.append(new_interest)
 
         new_annotation.created_by_id = relevant_annotation.created_by.id
@@ -312,6 +327,32 @@ class Annotation:
                action_description=f'update Annotation {relevant_annotation.id}',
                status=status)
 
+        # Annotations have a lot of properties that are managed by Appserver and shouldn't be pushed. We need to push
+        # only a small, curated set that are actually configuration items on the Topic Document.
+        properties_to_push = [SeeqNames.Properties.timezone,
+                              SeeqNames.Properties.is_ck_enabled]
+
+        added_or_updated_properties = [ScalarPropertyV1(name=k, value=v)
+                                       for k, v in self.definition_dict.items()
+                                       if k in properties_to_push and v is not None]
+
+        deleted_properties = [ScalarPropertyV1(name=k, value=v)
+                              for k, v in self.definition_dict.items()
+                              if k in properties_to_push and v is None]
+
+        if len(added_or_updated_properties) > 0:
+            safely(lambda: items_api.set_properties(id=relevant_annotation.id, body=added_or_updated_properties),
+                   action_description=f'set properties for Annotation {relevant_annotation.id}',
+                   status=status, additional_errors=[400])
+
+        if len(deleted_properties) > 0:
+            for deleted_property in deleted_properties:
+                safely(lambda: items_api.delete_property(id=relevant_annotation.id,
+                                                         property_name=deleted_property.name),
+                       action_description=f'delete property {deleted_property.name} '
+                                          f'for Annotation {relevant_annotation.id}',
+                       status=status, additional_errors=[400])
+
         item_map[f'{FIXUP_PREFIX} - {self.id}'] = types.SimpleNamespace(id=relevant_annotation.id, body=new_annotation)
 
     @staticmethod
@@ -378,8 +419,30 @@ class Annotation:
 
         _common.save_image_files(self.images, workbook_folder)
 
-    def _load(self, workbook_folder):
+        json_file = f'{self._get_annotation_file(workbook_folder)}.json'
+        json_dict = {
+            'Definition': self.definition_dict,
+        }
+        self._update_save_json_dict(json_dict)
+        with open(json_file, 'w', encoding='utf-8') as f:
+            json.dump(json_dict, f, indent=4, sort_keys=True)
+
+    def _update_save_json_dict(self, json_dict):
+        # Overridden in base class
+        pass
+
+    def _load(self, workbook_folder: str) -> dict:
         html_file = f'{self._get_annotation_file(workbook_folder)}.html'
+        json_file = f'{self._get_annotation_file(workbook_folder)}.json'
+
+        if os.path.exists(json_file):
+            with open(json_file, 'r', encoding='utf-8') as f:
+                json_dict = json.load(f)
+        else:
+            json_dict = dict()
+
+        if 'Definition' in json_dict:
+            self._definition = json_dict['Definition']
 
         with open(html_file, 'r', encoding='utf-8') as f:
             self.html = f.read()
@@ -392,6 +455,8 @@ class Annotation:
             with open(image_file, 'rb') as f:
                 self.images[image_id_tuple] = f.read()
 
+        return json_dict
+
     def add_image(self, *, filename=None, buffer=None, image_format=None, placement=None, just_src=False):
         """
         Add an image to the annotation.
@@ -503,6 +568,7 @@ class Report(Annotation):
 
     def __init__(self, worksheet):
         super().__init__(worksheet, 'Report')
+
         self.rendered_content_images = None
 
         self.date_ranges = dict()
@@ -543,12 +609,11 @@ class Report(Annotation):
             return
 
         for content in contents.content_items:
-            new_date_range = None
             if content.date_range is not None:
-                new_date_range = DateRange.pull(content.date_range, report=self, session=session, status=status)
+                new_date_range = DateRange.pull(content.date_range, report=self, annotation_output=annotation_output,
+                                                session=session, status=status)
                 self.date_ranges[new_date_range.id] = new_date_range
 
-            new_asset_selection = None
             if content.asset_selection is not None:
                 new_asset_selection = AssetSelection.pull(content.asset_selection, report=self, session=session,
                                                           status=status)
@@ -705,18 +770,16 @@ class Report(Annotation):
         super().save(workbook_folder, include_rendered_content=include_rendered_content,
                      pretty_print_html=pretty_print_html)
 
-        json_file = f'{self._get_annotation_file(workbook_folder)}.json'
-        json_dict = {
+        if include_rendered_content:
+            _render.save(self, workbook_folder)
+
+    def _update_save_json_dict(self, json_dict):
+        json_dict.update({
             'Schedule': self.schedule,
             'Date Ranges': [d.definition_dict for d in self.date_ranges.values()],
             'Asset Selections': [a.definition_dict for a in self.asset_selections.values()],
             'Content': [c.definition_dict for c in self.content.values()]
-        }
-        with open(json_file, 'w', encoding='utf-8') as f:
-            json.dump(json_dict, f, indent=4, sort_keys=True)
-
-        if include_rendered_content:
-            _render.save(self, workbook_folder)
+        })
 
     @staticmethod
     def load(worksheet, workbook_folder):
@@ -724,12 +787,8 @@ class Report(Annotation):
         report._load(workbook_folder)
         return report
 
-    def _load(self, workbook_folder):
-        super()._load(workbook_folder)
-
-        json_file = f'{self._get_annotation_file(workbook_folder)}.json'
-        with open(json_file, 'r', encoding='utf-8') as f:
-            json_dict = json.load(f)
+    def _load(self, workbook_folder: str) -> dict:
+        json_dict = super()._load(workbook_folder)
 
         self.schedule = json_dict['Schedule']
 
@@ -746,6 +805,8 @@ class Report(Annotation):
             new_content = Content(content_dict, self)
             self.content[new_content.id] = new_content
 
+        return json_dict
+
     def pull_rendered_content(self, session: Session, status: Status):
         _render.pull(session, self, status)
 
