@@ -10,6 +10,7 @@ import pandas as pd
 from deprecated import deprecated
 from mako.lookup import TemplateLookup
 from mako.template import Template
+
 from seeq import spy
 from seeq.sdk import *
 from seeq.spy import _common
@@ -27,7 +28,8 @@ from seeq.spy.workbooks._workstep import Workstep, AnalysisWorkstep, ExistingWor
 
 
 class Worksheet(Item):
-    _document: Union[Journal, Report]
+    _annotation: Union[Journal, Report]
+    _item_map_for_push: Optional[ItemMap]
 
     def __new__(cls, *args, **kwargs):
         if cls is Worksheet:
@@ -36,8 +38,8 @@ class Worksheet(Item):
 
         return object.__new__(cls)
 
-    def __init__(self, workbook, definition=None, add_defaults=True):
-        super().__init__(definition)
+    def __init__(self, workbook, definition=None, *, provenance=None, add_defaults=True):
+        super().__init__(definition, provenance=provenance)
 
         if workbook is None:
             raise SPyValueError("A Workbook is required to create a Worksheet")
@@ -47,12 +49,12 @@ class Worksheet(Item):
 
         if add_defaults:
             if self.workbook['Workbook Type'] == 'Analysis':
-                self._document = Journal(self)
+                self._annotation = Journal(self)
                 self.worksteps = dict()
                 workstep = AnalysisWorkstep(self)
                 self._definition['Current Workstep ID'] = workstep.id
             else:
-                self._document = Report(self)
+                self._annotation = Report(self)
 
     @property
     def url(self):
@@ -69,15 +71,16 @@ class Worksheet(Item):
         return f'{self.workbook.fqn} >> {self.name}'
 
     @property
+    @deprecated(reason='Use either journal (for AnalysisWorksheet) or report (for TopicDocument) property instead')
     def document(self):
-        return self._document
+        return self._annotation
 
     @staticmethod
-    def _instantiate(workbook, definition=None):
+    def _instantiate(workbook, definition=None, *, provenance=None):
         if workbook['Workbook Type'] == 'Analysis':
-            return AnalysisWorksheet(workbook, definition)
+            return AnalysisWorksheet(workbook, definition, provenance=provenance)
         else:
-            return TopicDocument(workbook, definition)
+            return TopicDocument(workbook, definition, provenance=provenance)
 
     @staticmethod
     def pull(item_id, *, workbook=None, extra_workstep_tuples=None, include_annotations=True, include_images=True,
@@ -97,7 +100,7 @@ class Worksheet(Item):
                 return
             else:
                 raise SPyRuntimeError(f'Worksheet with ID "{item_id}" does not exist')
-        worksheet = Worksheet._instantiate(workbook, definition)
+        worksheet = Worksheet._instantiate(workbook, definition, provenance=Item.PULL)
         worksheet.pull_worksheet(session, item_id, extra_workstep_tuples=extra_workstep_tuples,
                                  include_annotations=include_annotations, include_images=include_images, status=status)
         return worksheet
@@ -106,13 +109,11 @@ class Worksheet(Item):
                        status=None, include_annotations=True):
         status = Status.validate(status)
 
-        self._provenance = Item.PULL
-
         if include_images and not include_annotations:
             raise SPyValueError('Must specify include_images=False if you specify include_annotations=False')
 
         if include_annotations:
-            self.document.pull(session, include_images=include_images, status=status)
+            self._annotation.pull(session, include_images=include_images, status=status)
 
         link_url = _url.SeeqURL.parse(session.public_url)
         link_url.route = _url.Route.WORKBOOK_EDIT
@@ -129,6 +130,8 @@ class Worksheet(Item):
 
     def push(self, session: Session, pushed_workbook_id, item_map, datasource_output, existing_worksheet_identifiers,
              include_inventory, include_annotations, label=None, status: Status = None):
+        self._item_map_for_push = item_map
+
         existing_worksheet_id = None
 
         # After Integrated Security was introduced, we can no longer search for Worksheets using Data ID,
@@ -176,23 +179,23 @@ class Worksheet(Item):
             for workstep_id, workstep in self.worksteps.items():  # type: Workstep
                 referenced_items.extend(workstep.referenced_items)
 
-        referenced_items.extend(self.document.referenced_items)
+        referenced_items.extend(self._annotation.referenced_items)
 
         return referenced_items
 
     @property
     def referenced_worksteps(self):
-        return self.document.referenced_worksteps
+        return self._annotation.referenced_worksteps
 
     def find_workbook_links(self, session: Session, status: Status):
-        return self.document.find_workbook_links(session, status)
+        return self._annotation.find_workbook_links(session, status)
 
-    def find_unresolved_worksteps(self, item_map: ItemMap):
+    def find_unresolved_worksteps(self):
         # Unresolved Report workstep dependencies will be found when pushing the associated document
         dependencies_not_found = set()
-        for workstep_tuple in self.document.find_workstep_references(item_map=item_map):
+        for workstep_tuple in self._annotation.find_workstep_references(item_map=self._item_map_for_push):
             referenced_workbook_id, referenced_worksheet_id, referenced_workstep_id = workstep_tuple
-            if referenced_workstep_id not in item_map:
+            if referenced_workstep_id not in self._item_map_for_push:
                 dependencies_not_found.add(
                     'Workbook %s, Worksheet %s, Workstep %s' % (
                         referenced_workbook_id, referenced_worksheet_id, referenced_workstep_id))
@@ -211,8 +214,8 @@ class Worksheet(Item):
         with open(worksheet_json_file, 'w', encoding='utf-8') as f:
             json.dump(self._definition, f, indent=4)
 
-        self.document.save(workbook_folder, include_rendered_content=include_rendered_content,
-                           pretty_print_html=pretty_print_html)
+        self._annotation.save(workbook_folder, include_rendered_content=include_rendered_content,
+                              pretty_print_html=pretty_print_html)
 
     def _load(self, workbook_folder, worksheet_id):
         worksheet_json_file = Worksheet._get_worksheet_json_file(workbook_folder, worksheet_id)
@@ -220,12 +223,10 @@ class Worksheet(Item):
         with open(worksheet_json_file, 'r', encoding='utf-8') as f:
             self._definition = json.load(f)
 
-        self._provenance = Item.LOAD
-
         if self.workbook['Workbook Type'] == 'Analysis':
-            self._document = Journal.load(self, workbook_folder)
+            self._annotation = Journal.load(self, workbook_folder)
         else:
-            self._document = Report.load(self, workbook_folder)
+            self._annotation = Report.load(self, workbook_folder)
 
         self.worksteps = dict()
         workstep_files = glob.glob(os.path.join(workbook_folder, 'Worksheet_%s_Workstep_*.json' % worksheet_id))
@@ -236,7 +237,7 @@ class Worksheet(Item):
 
     @staticmethod
     def load_from_workbook_folder(workbook, workbook_folder, worksheet_id):
-        worksheet = Worksheet._instantiate(workbook)
+        worksheet = Worksheet._instantiate(workbook, provenance=Item.LOAD)
         worksheet._load(workbook_folder, worksheet_id)
         return worksheet
 
@@ -314,11 +315,19 @@ class AnalysisWorksheet(Worksheet):
         """
         Get/Set the raw HTML for the journal
         """
-        return self.document.html
+        return self.journal.html
 
     @html.setter
     def html(self, value):
-        self.document.html = value
+        self.journal.html = value
+
+    @property
+    def journal(self):
+        """
+        Returns the "Journal" object, which is an "Annotation" that houses the HTML and other metadata about the
+        Journal attached to this worksheet.
+        """
+        return self._annotation
 
     def workstep(self, name=None, create=True):
         if not name:
@@ -355,8 +364,8 @@ class AnalysisWorksheet(Worksheet):
             self._definition['Current Workstep ID'] = current_workstep_id
 
         if include_referenced_worksteps:
-            for workbook_id, worksheet_id, workstep_id in self.document.referenced_worksteps:
-                if isinstance(self.document, Journal) or worksheet_id == self.id:
+            for workbook_id, worksheet_id, workstep_id in self.journal.referenced_worksteps:
+                if isinstance(self.journal, Journal) or worksheet_id == self.id:
                     workstep_tuples_to_pull.add((workbook_id, worksheet_id, workstep_id))
 
         if extra_workstep_tuples:
@@ -390,7 +399,7 @@ class AnalysisWorksheet(Worksheet):
         quiet : bool, default False
             If True, suppresses progress output. Note that when status is
             provided, the quiet setting of the Status object that is passed
-            in takes precedent.
+            in takes precedence.
 
         status : spy.Status, optional
             If specified, the supplied Status object will be updated as the command
@@ -424,7 +433,7 @@ class AnalysisWorksheet(Worksheet):
         quiet : bool, default False
             If True, suppresses progress output. Note that when status is
             provided, the quiet setting of the Status object that is passed
-            in takes precedent.
+            in takes precedence.
 
         status : spy.Status, optional
             If specified, the supplied Status object will be updated as the command
@@ -494,15 +503,15 @@ class AnalysisWorksheet(Worksheet):
                status=status)  # type: WorksheetOutputListV1
 
         if include_annotations:
-            self.document.push(session, pushed_workbook_id, worksheet_output.id, item_map, datasource_output,
-                               push_images=True, label=label)
+            self.journal.push(session, pushed_workbook_id, worksheet_output.id, item_map, datasource_output,
+                              push_images=True, label=label)
 
         return worksheet_output
 
     def refresh_from(self, new_item, item_map: ItemMap, status: Status):
         super().refresh_from(new_item, item_map, status)
 
-        self.document.refresh_from(new_item.document, item_map, status)
+        self.journal.refresh_from(new_item.journal, item_map, status)
 
         for workstep_id, workstep in self.worksteps.items():
             workstep['ID'] = item_map[workstep_id]
@@ -897,11 +906,20 @@ class AnalysisWorksheet(Worksheet):
 
 class TopicDocument(Worksheet):
 
+    @property
+    def report(self):
+        """
+        Returns the "Report" object, which is an "Annotation" that houses the HTML and other metadata about the
+        TopicDocument. This object also includes dictionaries that contain the Content, DateRange and AssetSelection
+        objects.
+        """
+        return self._annotation
+
     def pull_rendered_content(self, session: Session, status: Status):
         status = Status.validate(status)
 
-        if self.document:
-            self.document.pull_rendered_content(session, status)
+        if self.report:
+            self.report.pull_rendered_content(session, status)
 
     def push(self, session: Session, pushed_workbook_id, item_map, datasource_output, existing_worksheet_identifiers,
              include_inventory, include_annotations, label=None, status: Status = None):
@@ -910,15 +928,15 @@ class TopicDocument(Worksheet):
                                         label=label, status=status)
 
         if include_annotations:
-            self.document.push(session, pushed_workbook_id, worksheet_output.id, item_map, datasource_output,
-                               push_images=True, label=label)
+            self.report.push(session, pushed_workbook_id, worksheet_output.id, item_map, datasource_output,
+                             push_images=True, label=label)
 
         return worksheet_output
 
     def refresh_from(self, new_item, item_map: ItemMap, status: Status):
         super().refresh_from(new_item, item_map, status)
 
-        self.document.refresh_from(new_item.document, item_map, status)
+        self.report.refresh_from(new_item.report, item_map, status)
 
     def render_template(self, text=None, filename=None, **kwargs):
         if filename:
@@ -948,13 +966,13 @@ class TopicDocument(Worksheet):
                                     f'class function, it needs to be decorated with @Asset.AssetSelection and return '
                                     f'a valid asset selection')
 
-            return self.document.get_embedded_content_html(display, date_range, size=size, shape=shape, width=width,
-                                                           height=height, scale=scale, selector=selector,
-                                                           asset_selection=asset_selection,
-                                                           summary_type=summary_type, summary_value=summary_value)
+            return self.report.get_embedded_content_html(display, date_range, size=size, shape=shape, width=width,
+                                                         height=height, scale=scale, selector=selector,
+                                                         asset_selection=asset_selection,
+                                                         summary_type=summary_type, summary_value=summary_value)
 
         def _plot(plot_function, date_range):
-            return self.document.add_plot_to_render(plot_function, date_range)
+            return self.report.add_plot_to_render(plot_function, date_range)
 
         folder_for_relative_paths = os.path.dirname(filename)
 
@@ -962,7 +980,7 @@ class TopicDocument(Worksheet):
         def _image(filename=None, *, buffer=None, image_format=None):
             if not os.path.isabs(filename):
                 filename = os.path.join(folder_for_relative_paths, filename)
-            return self.document.add_image(filename=filename, buffer=buffer, image_format=image_format, just_src=True)
+            return self.report.add_image(filename=filename, buffer=buffer, image_format=image_format, just_src=True)
 
         kwargs['display'] = _display
         kwargs['plot'] = _plot
@@ -975,23 +993,23 @@ class TopicDocument(Worksheet):
         """
         Get/Set the raw HTML for the topic document
         """
-        return self.document.html
+        return self.report.html
 
     @html.setter
     def html(self, value):
-        self.document.html = value
+        self.report.html = value
 
     @property
     def date_ranges(self):
-        return self.document.date_ranges
+        return self.report.date_ranges
 
     @property
     def content(self):
-        return self.document.content
+        return self.report.content
 
     @property
     def asset_selections(self):
-        return self.document.asset_selections
+        return self.report.asset_selections
 
     @property
     def schedule(self):
@@ -1021,11 +1039,11 @@ class TopicDocument(Worksheet):
         dict
             The current dictionary containing the 'Background' variable and 'Cron Schedule' for the document's schedule
         """
-        return self.document.schedule
+        return self.report.schedule
 
     @schedule.setter
     def schedule(self, value):
-        self.document.schedule = value
+        self.report.schedule = value
 
 
 def get_analysis_worksheet_from_url(url: str, include_archived: bool = False, quiet: Optional[bool] = False):
