@@ -355,8 +355,17 @@ class AnalysisWorkstep(Workstep):
         display_range = _common.get(duration_store, 'displayRange', default=dict())
         if not display_range:
             return None
-        start = _common.convert_to_timestamp(display_range['start'] * 1000000, 'UTC')
-        end = _common.convert_to_timestamp(display_range['end'] * 1000000, 'UTC')
+        try:
+            # Note: If the frontend stores the values as numeric, it's Epoch seconds rather than ns.
+            start_raw = display_range['start']
+            start_val = start_raw * 1_000_000 if not isinstance(start_raw, str) else start_raw
+            start = _common.convert_to_timestamp(start_val, 'UTC')
+            end_raw = display_range['end']
+            end_val = end_raw * 1_000_000 if not isinstance(end_raw, str) else end_raw
+            end = _common.convert_to_timestamp(end_val, 'UTC')
+        except BaseException as e:
+            raise SPyValueError(f'Error parsing Display Range. '
+                                f'Times must be numeric or valid ISO8601 strings.') from e
         return {'Start': start, 'End': end}
 
     def set_display_range(self, display_start_end, check_investigate=True, session: Optional[Session] = None):
@@ -422,8 +431,17 @@ class AnalysisWorkstep(Workstep):
         investigate_range = _common.get(duration_store, 'investigateRange', default=None)
         if investigate_range is None:
             return None
-        start = _common.convert_to_timestamp(investigate_range['start'] * 1000000, 'UTC')
-        end = _common.convert_to_timestamp(investigate_range['end'] * 1000000, 'UTC')
+        try:
+            # Note: If the frontend stores the values as numeric, it's Epoch seconds rather than ns.
+            start_raw = investigate_range['start']
+            start_val = start_raw * 1_000_000 if not isinstance(start_raw, str) else start_raw
+            start = _common.convert_to_timestamp(start_val, 'UTC')
+            end_raw = investigate_range['end']
+            end_val = end_raw * 1_000_000 if not isinstance(end_raw, str) else end_raw
+            end = _common.convert_to_timestamp(end_val, 'UTC')
+        except BaseException as e:
+            raise SPyValueError(f'Error parsing Investigate Range. '
+                                f'Times must be numeric or valid ISO8601 strings.') from e
         return {'Start': start, 'End': end}
 
     def set_investigate_range(self, investigate_start_end, check_display=True, session: Optional[Session] = None):
@@ -816,7 +834,7 @@ class AnalysisWorkstep(Workstep):
                 items.append(output_item)
 
         items_df = pd.DataFrame({'Name': pd.Series([], dtype=str), 'ID': pd.Series([], dtype=str)})
-        items_df = items_df.append(items, ignore_index=True).astype(object)
+        items_df = pd.concat([items_df, pd.DataFrame(items)], ignore_index=True).astype(object)
         return items_df
 
     def _get_scatter_plot_series(self):
